#!/usr/bin/perl
#
# Update.pl - Unified TTS chart update utility
#
# This program brings together the tts-list.cgi, rotatesamples.pl and
# daily.pl programs into one. The savings in runtime should be
# substantial, as only 1 pass is required to grab the accounts.
#
# Options:
#
#          -l    Perform list page update (tts-list.cgi)
#          -r    Perform sample page rotation (rotatesamples.pl)
#          -d    Perform daily maintenace (daily.pl)
#          -D    Perform daily maintenance in DEBUG mode
#          -s    Perform daily maintenance and echo output to STDOUT
#
# $Log: update.pl,v $
# Revision 2.8  1998/04/24 06:33:30  root
# Swapped 'require' order to put configuration.pl at the top. Needed for particularly
# braindead perl installations which don't point to the correct site_perl or perl/lib
# directory (the correct directory can be set in the @INC in configuration.pl)
#
# Revision 2.7  1998/04/23 05:23:24  root
# Scooped out the guts and inserted into 3 modules:
# listup.pl, samplerot.pl and daily.pl (all in the real dir)
# Makes things a lot simpler when one doesn't have to backtick "update.pl" from tts-master.cgi
#
# Revision 2.6  1998/03/25 11:53:45  root
# Fixed prior botch job on daily totals reporting in -d mode
#
# Revision 2.5  1998/03/24 20:53:37  root
# Cleaner method of moving visitor data to backup implemented
#
# Revision 2.4  1998/03/23 21:41:43  root
# Minor totals fix for refouts
#
# Revision 2.3  1998/03/23 12:35:29  root
# Fixed sneaky little multiline 'taint filter' problem with visitor data move
#
# Revision 2.2  1998/03/21 22:48:53  root
# Fixed tainted insecurity glitches and added result processing from the 'move' function
#
# Revision 2.1  1998/03/21 18:11:51  root
# Minor mailout size fix
#
# Revision 2.0  1998/03/20 22:53:53  root
# Version updated to 2.0 for TTSv2.0 release
#
# Revision 1.3  1998/03/20 22:40:02  root
# Moved mail to an internal routine (makes life easier for pushbutton updates)
#
# Revision 1.2  1998/03/10 22:58:20  root
# RefOut mojifikashuns (hukt on foniks werkt for mi!)
#
# Revision 1.1  1998/02/28 14:55:43  root
# Initial revision
#

require "./configuration.pl";
require "getopts.pl";

$|=1;

$rcsid = '$Id: update.pl,v 2.8 1998/04/24 06:33:30 root Exp root $';

$tmpmail = "$Home/tts.mail";

Getopts('lrdDs');

if($opt_d + $opt_r + $opt_l == 0) {
  die("
Usage: update.pl [-l] [-r] [-d [-D] [-s]]

Options:

-l\tList refresh (regenerate output from templates)
-r\tRotate sample image pages
-d\tDaily list reset (flush totals)
-D\tUsed with -d, run the daily reset in 'debug mode'
-s\tUsed with -d, run the daily reset and send output to STDOUT

 Copyright 1998 by Rick Franchuk - TranSpecT Consulting
");
}

if($opt_l==1) {
  require "listup.pl";

  $res = do_list_update();
  dodie($res) if($res ne "");
}

if($opt_r==1) {
  require "samplerot.pl";

  $res = do_rotate_samples();
  dodie($res) if($res ne "");
}

if($opt_d==1 || $opt_D==1) {
  require "daily.pl";

  $res = do_daily_reset();
  dodie($res) if($res ne "");
}
